package it.andynaz.utils;

import java.io.*;


/**
 * Metodi utili per una elementare gestione di file.
 * 
 * @author andynaz
 * @version 0.2 2012/07/20
 */
public class UtilsIO{
	// nasconde il costruttore
	private UtilsIO(){}
	
	/**
	 * Legge il contenuto di un file di testo.
	 * 
	 * Il contenuto del file viene restituito sotto forma di
	 * <code>StringBuilder</code>.
	 * 
	 * In caso di errore di I/O, viene interrotta la lettura.
	 * 
	 * @param file file da cui leggere
	 * @return uno <code>StringBuilder</code> con il contenuto del file
	 * 
	 * @throws FileNotFoundException se il file non esiste
	 * 
	 * @since 0.1
	 */	
	public static StringBuilder readFromFile(File file) throws FileNotFoundException{
		StringBuilder content = new StringBuilder(500);
		try{
			String line = null;
			FileReader reader = new FileReader(file);
			BufferedReader br = new BufferedReader(reader);
			while( (line=br.readLine())!=null ){
				content.append(line);
				content.append('\n');
			}
		} catch(IOException e){
			Utils.printException("LocalUtils::readFromFile", e);
		}
		return content;
	}
	
	
	/**
	 * Scrive una stringa in un file di testo.
	 * 
	 * Il metodo cattura tutte le eccezioni, e viene restituito un booleano con
	 * l'esito dell'operazione.
	 * 
	 * In caso di errore di I/O, viene interrotta la scrittura.
	 * 
	 * @param file file in cui scrivere
	 * @param content contenuto da scrivere
	 * @return <tt>true</tt> se la scrittura &egrave; avvenuta in modo corretto,
	 *   <tt>false</tt> altrimenti
	 * 
	 * @since 0.1
	 */
	public static boolean writeToFile(File file, String content){
		try{
			StringReader reader = new StringReader(content);
			FileWriter writer = new FileWriter(file);
			char[] buff = new char[512];
			int len = buff.length;
			while( (len=reader.read(buff, 0, len))!=-1 ){
				writer.write(buff, 0, len);
				writer.flush();
			}
			writer.close();
			reader.close();
			return true;
		} catch(IOException e){
			Utils.printException("LocalUtils::writeToFile", e);
			return false;
		}
	}
	
	
	/**
	 * Copia il contenuto di un <tt>InputStream</tt> in un <tt>OutputStream</tt>.
	 * 
	 * Viene tentato prima lo spostamento con il metodo
	 * <code>File.renameTo(File)</code>; se questo non riesce effettua una copia
	 * del contenuto del file nella nuova locazione e cancella il file originale.
	 * 
	 * @param in stream da cui leggere
	 * @param out stream in cui scrivere
	 * @param errMsg messaggio di eventuale errore: viene aggiunto all'eccezione
	 *   (<tt>IOException</tt>) in caso di errore
	 * 
	 * @throws IOException in caso di errore di I/O
	 * 
	 * @since 0.1
	 */
	public static void cp(InputStream in, OutputStream out, String errMsg) throws IOException{
		try{
			byte[] buff = new byte[512];
			int len;
			while( (len=in.read(buff))!=-1 ){
				out.write(buff, 0, len);
			}
			out.flush();
			out.close();
			in.close();
		} catch(IOException e){
			if( Utils.isEmpty(errMsg) )
				throw e;
			else
				throw new IOException(errMsg);
		}
	}
	
	
	/**
	 * Sposta un file in un'altra posizione.
	 * 
	 * Viene tentato prima lo spostamento con il metodo
	 * <code>File.renameTo(File)</code>; se questo non riesce effettua una copia
	 * del contenuto del file nella nuova locazione e cancella il file originale.
	 * 
	 * @param from file da spostare
	 * @param to nuova locazione del file
	 * @param errMsg messaggio di eventuale errore: viene aggiunto all'eccezione
	 *   (<tt>IOException</tt>) in caso di errore
	 * @return <tt>true</tt> se lo spostamento è andato a buon fine,
	 *   <tt>false</tt> altrimenti
	 * 
	 * @throws IOException in caso di errore di I/O
	 * 
	 * @since 0.1
	 */
	public static boolean mv(File from, File to, String errMsg) throws IOException{
		if( from.renameTo(to) ){
			return true;
		}
		
		try{
			FileInputStream fileIS = new FileInputStream(from);
			FileOutputStream fileOS = new FileOutputStream(to);
			byte[] buff = new byte[512];
			int len;
			while( (len=fileIS.read(buff))!=-1 ){
				fileOS.write(buff, 0, len);
				fileOS.flush();
			}
			fileOS.close();
			fileIS.close();
			
			return from.delete();
		} catch(IOException e){
			if( Utils.isEmpty(errMsg) )
				throw e;
			else
				throw new IOException(errMsg);
		}
	}
}
